package javax.swing;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.security.PrivilegedAction;
import java.util.EventListener;
import java.awt.event.ActionEvent;
import java.security.AccessController;
import java.util.concurrent.locks.ReentrantLock;
import java.awt.event.ActionListener;
import java.security.AccessControlContext;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.atomic.AtomicBoolean;
import javax.swing.event.EventListenerList;
import java.io.Serializable;

public class Timer implements Serializable
{
    protected EventListenerList listenerList;
    private final transient AtomicBoolean notify;
    private volatile int initialDelay;
    private volatile int delay;
    private volatile boolean repeats;
    private volatile boolean coalesce;
    private final transient Runnable doPostEvent;
    private static volatile boolean logTimers;
    private final transient Lock lock;
    transient TimerQueue.DelayedTimer delayedTimer;
    private volatile String actionCommand;
    private transient volatile AccessControlContext acc;
    
    public Timer(int n, ActionListener actionListener) {
        this.listenerList = new EventListenerList();
        this.notify = new AtomicBoolean(false);
        this.repeats = true;
        this.coalesce = true;
        this.lock = new ReentrantLock();
        this.delayedTimer = null;
        this.acc = AccessController.getContext();
        this.delay = n;
        this.initialDelay = n;
        this.doPostEvent = new Timer.DoPostEvent();
        if (actionListener != null) {
            this.addActionListener(actionListener);
        }
    }
    
    final AccessControlContext getAccessControlContext() {
        if (this.acc == null) {
            throw new SecurityException("Timer is missing AccessControlContext");
        }
        return this.acc;
    }
    
    public void addActionListener(ActionListener l) {
        this.listenerList.add(ActionListener.class, l);
    }
    
    public void removeActionListener(ActionListener l) {
        this.listenerList.remove(ActionListener.class, l);
    }
    
    public ActionListener[] getActionListeners() {
        return (ActionListener[])this.listenerList.getListeners(ActionListener.class);
    }
    
    protected void fireActionPerformed(ActionEvent actionEvent) {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ActionListener.class) {
                ((ActionListener)listenerList[i + 1]).actionPerformed(actionEvent);
            }
        }
    }
    
    public <T extends EventListener> T[] getListeners(Class<T> t) {
        return this.listenerList.getListeners(t);
    }
    
    private TimerQueue timerQueue() {
        return TimerQueue.sharedInstance();
    }
    
    public static void setLogTimers(boolean logTimers) {
        Timer.logTimers = logTimers;
    }
    
    public static boolean getLogTimers() {
        return Timer.logTimers;
    }
    
    public void setDelay(int n) {
        if (n < 0) {
            throw new IllegalArgumentException("Invalid delay: " + n);
        }
        this.delay = n;
        return;
    }
    
    public int getDelay() {
        return this.delay;
    }
    
    public void setInitialDelay(int n) {
        if (n < 0) {
            throw new IllegalArgumentException("Invalid initial delay: " + n);
        }
        this.initialDelay = n;
        return;
    }
    
    public int getInitialDelay() {
        return this.initialDelay;
    }
    
    public void setRepeats(boolean repeats) {
        this.repeats = repeats;
    }
    
    public boolean isRepeats() {
        return this.repeats;
    }
    
    public void setCoalesce(boolean coalesce) {
        int coalesce2 = this.coalesce;
        this.coalesce = coalesce;
        if (coalesce2 == 0 && this.coalesce) {
            this.cancelEvent();
        }
    }
    
    public boolean isCoalesce() {
        return this.coalesce;
    }
    
    public void setActionCommand(String actionCommand) {
        this.actionCommand = actionCommand;
    }
    
    public String getActionCommand() {
        return this.actionCommand;
    }
    
    public void start() {
        this.timerQueue().addTimer(this, (long)this.getInitialDelay());
    }
    
    public boolean isRunning() {
        return this.timerQueue().containsTimer(this);
    }
    
    public void stop() {
        this.getLock().lock();
        try {
            this.cancelEvent();
            this.timerQueue().removeTimer(this);
        }
        finally {
            this.getLock().unlock();
        }
    }
    
    public void restart() {
        this.getLock().lock();
        try {
            this.stop();
            this.start();
        }
        finally {
            this.getLock().unlock();
        }
    }
    
    void cancelEvent() {
        this.notify.set(false);
    }
    
    void post() {
        if (this.notify.compareAndSet(false, true) || !this.coalesce) {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    SwingUtilities.invokeLater(Timer.this.doPostEvent);
                    return null;
                }
            }, this.getAccessControlContext());
        }
    }
    
    Lock getLock() {
        return this.lock;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        this.acc = AccessController.getContext();
        objectInputStream.defaultReadObject();
    }
    
    private Object readResolve() {
        Timer timer = new Timer(this.getDelay(), null);
        timer.listenerList = this.listenerList;
        timer.initialDelay = this.initialDelay;
        timer.delay = this.delay;
        timer.repeats = this.repeats;
        timer.coalesce = this.coalesce;
        timer.actionCommand = this.actionCommand;
        return timer;
    }
    
    class DoPostEvent implements Runnable
    {
        @Override
        public void run() {
            if (Timer.logTimers) {
                System.out.println("Timer ringing: " + Timer.this);
            }
            if (Timer.this.notify.get()) {
                Timer.this.fireActionPerformed(new ActionEvent(Timer.this, 0, Timer.this.getActionCommand(), System.currentTimeMillis(), 0));
                if (Timer.this.coalesce) {
                    Timer.this.cancelEvent();
                }
            }
        }
        
        Timer getTimer() {
            return Timer.this;
        }
    }
}
